"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = build;

var _getAllBobs = _interopRequireDefault(require("../utils/getAllBobs"));

var _browserify = _interopRequireDefault(require("../tasks/browserify"));

var _sass = _interopRequireDefault(require("../tasks/sass"));

var _babel = _interopRequireDefault(require("../tasks/babel"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const definedTasks = ['babel', 'browserify', 'html', 'nodemon', 'sass', 'copy', 'static', 'reload'];
const noBuild = ['nodemon', 'static'];
let tasks = {};

function build(config) {
  const {
    dev,
    log,
    relative,
    chalk,
    globalIgnore
  } = config;
  process.env.NODE_ENV = dev ? 'development' : 'production';
  const allBobs = (0, _getAllBobs.default)(config);
  log.title('Build'); //, allBobs

  let allTasks = [];
  Object.keys(allBobs).forEach(key => {
    if (definedTasks.indexOf(key) < 0) {
      log.error(`Unknown task "${key}"`, allBobs[key]);
      return process.exit(1);
    }

    if (noBuild.indexOf(key) >= 0) return;
    if (!tasks[key]) tasks[key] = require(`../tasks/${key}`);
    allBobs[key].forEach(bundle => {
      if (key !== 'reload') allTasks.push(tasks[key]({ ...bundle,
        dev,
        log,
        relative,
        chalk,
        globalIgnore
      }));
    });
  });
  return Promise.all(allTasks).then(() => ({
    bob: allBobs,
    tasks
  })); //.catch(e => log.error(e))
}

module.exports = exports.default;