"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = browserifyTask;

var _path = _interopRequireDefault(require("path"));

var _gulp = _interopRequireDefault(require("gulp"));

var _gulpBro = _interopRequireDefault(require("gulp-bro"));

var _gulpRename = _interopRequireDefault(require("gulp-rename"));

var _babelify = _interopRequireDefault(require("babelify"));

var _gulpUglify = _interopRequireDefault(require("gulp-uglify"));

var _gulpIf = _interopRequireDefault(require("gulp-if"));

var _config = _interopRequireDefault(require("../babel/config"));

var _fileExists = _interopRequireDefault(require("../utils/fileExists"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function browserifyTask(config) {
  const {
    src,
    dest,
    dev = false,
    log,
    relative,
    chalk
  } = config;

  const srcDir = _path.default.dirname(src);

  const destDir = _path.default.dirname(dest);

  const destFile = _path.default.basename(dest);

  const extensions = ['.js', '.jsx', '.json', '.ts', '.tsx'];

  if (!(0, _fileExists.default)(src)) {
    log.error('browserify', `File doesn't exist: ${src}`);
    return Promise.resolve();
  }

  return new Promise((resolve, reject) => {
    return _gulp.default.src(src, {
      read: false,
      // recommended option for gulp-bro
      allowEmpty: true
    }).pipe((0, _gulpBro.default)({
      extensions,
      debug: dev,
      // Source maps
      transform: [_babelify.default.configure({
        extensions,
        ...(0, _config.default)(config)
      })],
      // Resolve require paths for client source
      // For server-side babel, define NODE_PATH
      paths: [_path.default.resolve(srcDir)]
    })).pipe((0, _gulpIf.default)(!dev, (0, _gulpUglify.default)())).pipe((0, _gulpRename.default)(destFile)).pipe(_gulp.default.dest(destDir)).on('error', function (e) {
      log.error('browserify', e.message);
      this.emit('end');
      reject();
    }).on('end', () => {
      log('browserify', `${relative(src)} -> ${chalk.green(relative(dest))}`);
      resolve();
    });
  });
}

module.exports = exports.default;