"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = sassTask;

var _path = _interopRequireDefault(require("path"));

var _gulp = _interopRequireDefault(require("gulp"));

var _gulpRename = _interopRequireDefault(require("gulp-rename"));

var _gulpSass = _interopRequireDefault(require("gulp-sass"));

var _gulpSourcemaps = _interopRequireDefault(require("gulp-sourcemaps"));

var _gulpCleanCss = _interopRequireDefault(require("gulp-clean-css"));

var _gulpAutoprefixer = _interopRequireDefault(require("gulp-autoprefixer"));

var _gulpIf = _interopRequireDefault(require("gulp-if"));

var _fileExists = _interopRequireDefault(require("../utils/fileExists"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function sassTask({
  src,
  dest,
  bundleRoot,
  dev = false,
  log,
  relative,
  chalk
}) {
  const rootSrc = _path.default.join(bundleRoot, 'src');

  const destDir = _path.default.dirname(dest);

  const destFile = _path.default.basename(dest);

  if (!(0, _fileExists.default)(src)) {
    log.error('sass', `File doesn't exist: ${src}`);
    return Promise.resolve();
  }

  return new Promise((resolve, reject) => {
    _gulp.default.src(src, {
      allowEmpty: true
    }).pipe((0, _gulpIf.default)(dev, _gulpSourcemaps.default.init())).pipe((0, _gulpSass.default)({
      keepSpecialComments: false,
      // Resolve require paths for client source
      includePaths: [rootSrc],
      //relativeTo: root, //'./app',
      processImport: false // ?

    })).on('error', function (e) {
      log.error('sass', e.message);
      this.emit('end');
      reject();
    }).pipe((0, _gulpAutoprefixer.default)({
      overrideBrowserslist: ['last 2 versions', 'IE 10', '> 1%'],
      cascade: false
    })).pipe((0, _gulpIf.default)(!dev, (0, _gulpCleanCss.default)())).pipe((0, _gulpRename.default)(destFile)).pipe((0, _gulpIf.default)(dev, _gulpSourcemaps.default.write())).pipe(_gulp.default.dest(destDir)).on('end', () => {
      log('sass', `${relative(src)} -> ${chalk.green(relative(dest))}`);
      resolve();
    });
  });
}

module.exports = exports.default;