"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getAllBobs;

var _path = _interopRequireDefault(require("path"));

var _getBundles = _interopRequireDefault(require("./getBundles"));

var _defaults = _interopRequireDefault(require("../defaults"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getAllBobs(config) {
  const {
    args,
    relative
  } = config;
  const bundles = (0, _getBundles.default)(config);
  let allBobs = {};
  bundles.forEach(bundle => {
    const root = bundle.root;
    const relativeRoot = relative(root); // If project dirs specified

    if (args.length && args.indexOf(relativeRoot ? relativeRoot : '.') < 0) return;
    if (!bundle.json || !bundle.json.bob) return; // Live reload default

    let {
      livereload = false,
      includePolyfill = false,
      ...bob
    } = bundle.json.bob;
    bob = (0, _defaults.default)(bob);
    if (bob.reload || typeof bob.livereload === 'undefined' && (bob.static || bob.nodemon) // Server
    && bob.html // HTML is needed for live reload client
    ) livereload = true;
    Object.keys(bob).forEach(task => {
      const bobWithRelativePaths = [];
      bob[task].forEach(b => {
        // Append relative root to these keys
        ['src', 'dest', 'watch'].forEach(key => {
          if (!b[key] || typeof b[key] !== 'string') return;
          b[key] = _path.default.join(relativeRoot, b[key]);
        }); // Root path to resolve requires for browserify, babel and sass

        b.root = b.root || root;
        b.bundleRoot = root; // Each bundle can set livereload off

        b.livereload = typeof b.livereload !== 'undefined' ? b.livereload : livereload;

        if (task === 'browserify' || task === 'babel') {
          b.includePolyfill = typeof b.includePolyfill !== 'undefined' ? b.includePolyfill : includePolyfill;
        }

        bobWithRelativePaths.push(b);
      });
      if (!allBobs[task]) allBobs[task] = [];
      allBobs[task].push(...bobWithRelativePaths);
    });
  });
  return allBobs;
}

module.exports = exports.default;