"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getBundles;

var _path = _interopRequireDefault(require("path"));

var _glob = _interopRequireDefault(require("glob"));

var _fileExists = _interopRequireDefault(require("./fileExists"));

var _getPackageJSON = _interopRequireDefault(require("./getPackageJSON"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getBundles(config) {
  const {
    root,
    log,
    options
  } = config;
  const bundles = [];
  const paths = [root];
  paths.forEach(thisPath => {
    const packageName = 'package.json';
    const globPath = options.sub ? `${thisPath}/**/${packageName}` // Include sub folders
    : `${thisPath}/${packageName}`;

    const packages = _glob.default.sync(globPath, {
      ignore: ['**/.git/**', '**/_*/**', `**/node_modules/**`]
    });

    if (!packages.length && !options.sub) {
      packages.push(_path.default.join(thisPath, 'package.json'));
    }

    packages.forEach(packagePath => {
      const root = _path.default.dirname(packagePath);

      const bundle = {
        root,
        json: (0, _getPackageJSON.default)({
          root,
          ...config
        })
      };
      bundles.push(bundle);
    });
  });

  if (!bundles.length) {
    log.error('No projects found', paths);
    return process.exit(1);
  }

  return bundles;
}

module.exports = exports.default;