"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getConfig;

var _path = _interopRequireDefault(require("path"));

var _minimist = _interopRequireDefault(require("minimist"));

var _readlineSync = _interopRequireDefault(require("readline-sync"));

var _chalk = _interopRequireDefault(require("chalk"));

var _logger = _interopRequireDefault(require("./logger"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getConfig({
  commands,
  defaultCommand
}) {
  const options = (0, _minimist.default)(process.argv.slice(2));
  const args = options._; // Log/prompt

  const log = (0, _logger.default)({
    verbose: !options.quiet
  });
  const yesno = _readlineSync.default.keyInYN;
  const question = _readlineSync.default.question; // Command

  let command = defaultCommand;

  if (args[0]) {
    if (commands.indexOf(args[0]) < 0) {
      log.error(`Unknown command "${args[0]}"`);
      return process.exit(1);
    }

    command = args[0];
    args.shift();
  } // Paths


  const root = process.cwd();

  const relative = name => _path.default.relative(root, name);

  const config = {
    root,
    relative,
    command,
    options,
    args,
    log,
    yesno,
    question,
    chalk: _chalk.default,
    globalIgnore: ['!**/_*/**', '!**/.git/**', '!**/node_modules/**']
  };
  return config;
}

module.exports = exports.default;